@@ -35,12 +35,12 @@ __all__ = ["CaselessList", "CaselessDict", "CaselessWeakValueDict", "LoopList",
 
 __docformat__ = "restructuredtext"
 
-import weakref
 import copy
-import numpy
 import time
+import weakref
 import operator
 
+
 class CaselessList(list):
     """A case insensitive lists that has some caseless methods. Only allows 
     strings as list members. Most methods that would normally return a list, 
@@ -1054,8 +1054,9 @@ class ArrayBuffer(object):
         shape = list(self.__buffer.shape)
         shape[0] = newlen
         try:
-            self.buffer.resize(shape) #first try to resize in-place
+            self.__buffer.resize(shape) #first try to resize in-place
         except:
+            import numpy
             self.__buffer = numpy.resize(self.__buffer, shape) #if not possible, do it by copying
         self.__bsize = self.__buffer.shape[0]
     
