@@ -381,11 +381,7 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
             return self._findAbsoluteModelClass(self.getModel())
         
     def _findAbsoluteModelClass(self, absolute_name):
-        obj = taurus.Manager().findObject(absolute_name)
-        if obj is None:
-            return None
-        else:
-            return obj.__class__
+        return taurus.Manager().findObjectClass(absolute_name)
         
     def _findRelativeModelClass(self, relative_name):
         parent_widget = self.getParentTaurusComponent()
@@ -652,6 +648,9 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
         if cls is None:
             self._attached = False
             #self.trace("Failed to attach: Model class not found")
+        elif self.modelName == '':
+            self._attached = False
+            self.modelObj = None
         else:
             try:
                 self.modelObj = taurus.Manager().getObject(cls, self.modelName)
@@ -1563,9 +1562,11 @@ class TaurusBaseWritableWidget(TaurusBaseWidget):
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
     
     def getModelClass(self):
+        '''reimplemented from :class:`TaurusBaseWidget`'''
         return TaurusAttribute
 
     def isReadOnly(self):
+        '''reimplemented from :class:`TaurusBaseWidget`'''
         return False
     
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
@@ -1573,27 +1574,55 @@ class TaurusBaseWritableWidget(TaurusBaseWidget):
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 
     def setAutoApply(self, auto):
+        '''
+        Sets autoApply mode. In autoApply mode, the widget writes the value
+        automatically whenever it is changed by the user (e.g., when 
+        :meth:`valueChanged` is called). If False, a value changed just 
+        flags a "pending operation" which needs to be applied manually by 
+        the user before the value gets written.
+        
+        :param auto: (bool) True for setting autoApply mode. False for disabling
+        '''
         self._autoApply = auto
         
     def getAutoApply(self):
+        '''whether autoApply mode is enabled or not.
+        
+        :return: (bool) 
+        '''
         return self._autoApply
 
     def resetAutoApply(self):
+        '''resets the autoApply mode (i.e.: sets it to False)'''
         self.setAutoApply(False)
         
     def setForcedApply(self, forced):
+        '''Sets the forcedApply mode. In forcedApply mode, values are written even 
+        if there are not pending operations (e.g. even if the displayed value is 
+        the same as the currently applied one).
+        
+        .. seealso: :meth:`forceApply` and :meth:`writeValue`
+        
+        :param forced: (bool) True for setting forcedApply mode. False for disabling
+        '''
         self._forcedApply = forced
         
     def getForcedApply(self):
+        '''whether forcedApply mode is enabled or not.
+        
+        :return: (bool) 
+        '''
         return self._forcedApply
 
     def resetForcedApply(self):
+        '''resets the forcedApply mode (i.e.: sets it to False)'''
         self.setForcedApply(False)
         
     def valueChanged(self, *args):
         '''Subclasses should connect some particular signal to this method for
         indicating that something has changed.
-        e.g., a QLineEdit should connect its "textChanged" signal...'''
+        e.g., a QLineEdit should connect its "textChanged" signal...
+        '''
         self.emitValueChanged()
         if self._autoApply:
             self.writeValue()
@@ -1601,7 +1630,12 @@ class TaurusBaseWritableWidget(TaurusBaseWidget):
     def writeValue(self, forceApply=False):
         '''Writes the value to the attribute, either by applying pending
         operations or (if the ForcedApply flag is True), it writes directly when
-        no operations are pending'''
+        no operations are pending
+        
+        :param forceApply: (bool) If True, it behaves as in forceApply mode 
+                           (even if the forceApply mode is disabled by 
+                           :meth:`setForceApply`)
+        '''
         
         if self.hasPendingOperations():
             self.safeApplyOperations()
@@ -1616,7 +1650,10 @@ class TaurusBaseWritableWidget(TaurusBaseWidget):
     def forceApply(self):
         '''It (re)applies the value regardless of pending operations.
         WARNING: USE WITH CARE. In most cases what you need is to make sure
-        that pending operations are properly created, not calling this method'''
+        that pending operations are properly created, not calling this method
+        
+        .. seealso: :meth:`forceApply` and :meth:`writeValue`
+        '''
         try:
             v = self.getValue()
             op = WriteAttrOperation(self.getModelObj(), v, 
@@ -1629,10 +1666,12 @@ class TaurusBaseWritableWidget(TaurusBaseWidget):
             self.traceback()
         
     def handleEvent(self, src, evt_type, evt_value):
+        '''reimplemented from :class:`TaurusBaseWidget`'''
         if evt_type in (TaurusEventType.Change, TaurusEventType.Periodic):
             self.emitValueChanged()
     
     def postAttach(self):
+        '''reimplemented from :class:`TaurusBaseWidget`'''
         TaurusBaseWidget.postAttach(self)
         if self.isAttached():
             try:
@@ -1646,6 +1685,7 @@ class TaurusBaseWritableWidget(TaurusBaseWidget):
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
     
     def resetPendingOperations(self):
+        '''reimplemented from :class:`TaurusBaseWidget`'''
         if self.isAttached():
             try:
                 v = self.getModelValueObj().w_value
@@ -1656,6 +1696,7 @@ class TaurusBaseWritableWidget(TaurusBaseWidget):
         self.updateStyle()
 
     def updatePendingOperations(self):
+        '''reimplemented from :class:`TaurusBaseWidget`'''
         model = self.getModelObj()
         try:
             model_value = model.getValueObj().w_value
@@ -1672,15 +1713,35 @@ class TaurusBaseWritableWidget(TaurusBaseWidget):
         self.updateStyle()
         
     def getOperationCallbacks(self):
+        '''returns the operation callbacks (i.e., a sequence of methods that will be called after an operation is executed
+           (this default implementation it returns an empty list).
+           
+        :return: (sequence<callable>)   
+        '''
         return []
 
     def getValue(self):
+        '''
+        This method must be implemented in derived classes to return 
+        the value to be written. Note that this may differ 
+        from the displayed value (e.g. for a numeric value being 
+        edited by a QLineEdit-based widget, the displayed value will 
+        be a string while getValue will return a number)
+        '''
         raise NotImplementedError("Not allowed to call TaurusBaseWritableWidget.getValue()")
 
     def setValue(self, v):
+        '''
+        This method must be implemented in derived classes to provide 
+        a (widget-specific) way of updating the displayed value based 
+        on a given attribute value
+        
+        :param v: The attribute value 
+        '''
         raise NotImplementedError("Not allowed to call TaurusBaseWritableWidget.setValue()")
 
     def updateStyle(self):
+        '''reimplemented from :class:`TaurusBaseWidget`'''
         if self._autoTooltip:
             toolTip = self.getFormatedToolTip()
             if self.hasPendingOperations():
@@ -1711,6 +1772,7 @@ class TaurusBaseWritableWidget(TaurusBaseWidget):
     
     @classmethod
     def getQtDesignerPluginInfo(cls):
+        '''reimplemented from :class:`TaurusBaseWidget`'''
         ret = TaurusBaseWidget.getQtDesignerPluginInfo()
         ret['group'] = 'Taurus Input'
         return ret
