@@ -104,7 +104,7 @@ class TaurusCurveItem(CurveItem, TaurusBaseComponent):
         if xvalue is None:
             xvalue = numpy.arange(len(yvalue))
         
-        self.setData(xvalue, yvalue)
+        self.set_data(xvalue, yvalue)
         p = self.plot()
         if p is not None: 
             p.replot()
