@@ -274,8 +274,8 @@ class TaurusDevicePanel(TaurusWidget):
             self.warning('TaurusDevicePanel accepts only Device models')
             return
         try:
-            if self.getModel(): self.detach()
             taurus.Device(model).ping()
+            if self.getModel(): self.detach() #Do not dettach previous model before pinging the new one (fail message will be shown at except: clause)
             TaurusWidget.setModel(self,model)
             self.setWindowTitle(str(model).upper())
             model = self.getModel()
@@ -331,6 +331,9 @@ class TaurusDevicePanel(TaurusWidget):
     def detach(self):
         self.trace('In TaurusDevicePanel(%s).detach()'%self.getModel())
         _detached = []
+        #long imports to avoid comparison problems in the isinstance below 
+        import taurus.qt.qtgui.container
+        import taurus.qt.qtgui.base
         def detach_recursive(obj):
             if obj in _detached: return
             if isinstance(obj,taurus.qt.qtgui.container.TaurusBaseContainer):
@@ -346,6 +349,14 @@ class TaurusDevicePanel(TaurusWidget):
                     self.warning(traceback.format_exc())                    
             _detached.append(obj)
         detach_recursive(self)
+        try:
+            self._label.setText('')
+            self._state.setModel('')
+            if hasattr(self,'_statelabel'): self._statelabel.setModel('')
+            self._status.setModel('')
+            self._image.setPixmap(Qt.QPixmap())
+        except:
+            self.warning(traceback.format_exc())
         
     def get_attrs_form(self,device,form=None,filters=None,parent=None):
         filters = filters or get_regexp_dict(TaurusDevicePanel._attribute_filter,device,['.*'])
@@ -519,40 +530,41 @@ class TaurusDevPanel(TaurusMainWindow):
 
 def TaurusDevicePanelMain():
     '''A launcher for TaurusDevicePanel.'''
-    #!/usr/bin/python
     import sys
     from taurus.qt.qtgui.application import TaurusApplication
     from taurus.core.util import argparse
     
     parser = argparse.get_taurus_parser()
-    parser.set_usage("%prog [options] devname [attrs]")
+    parser.set_usage("%prog [options] [devname [attrs]]")
     parser.set_description("Taurus Application inspired in Jive and Atk Panel")
     parser.add_option("", "--config-file", dest="config_file", default=None,
-                  help="launch a wizard for creating a new TaurusGUI application")
+                  help="load a config file (TODO: document this option)") 
                       
     app = TaurusApplication(cmd_line_parser=parser,app_name="TaurusDevicePanel",
                             app_version=taurus.Release.version)
     args = app.get_command_line_args()
     options = app.get_command_line_options()
     
-    app.setLogLevel(taurus.Debug)
-    
     w = TaurusDevicePanel()
     w.show()
-    if options.tango_host is None:
-        options.tango_host = taurus.Database().getNormalName()
-    try: w.setTangoHost(options.tango_host)
-    except: pass
     
+    if len(args) == 0:
+        from taurus.qt.qtgui.panel import TaurusModelChooser
+        models, ok = TaurusModelChooser.modelChooserDlg(w, 
+                                       selectables = [TaurusElementType.Member],
+                                       singleModel= True )
+        model = models[0] if ok and models else None
+        filters = ''
+    else:
+        model = args[0]
+        filters = args[1:]
+
     if options.config_file is not None:
         w.loadConfigFile(options.config_file)
-    else:
-        w.setAttributeFilters({args[0]:args[1:]})
-        
-    if len(args)<1:
-        parser.print_help() #@todo use modelchooser instead of printing the help
-        return
-    w.setModel(args[0])
+    elif model and filters:
+        w.setAttributeFilters({model:filters})
+
+    w.setModel(model)
     
     sys.exit(app.exec_())             
 
